/*
 * deviced
 *
 * Copyright (c) 2012 - 2013 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include <libgdbus/dbus-system.h>

#include "log.h"
#include "list.h"
#include "common.h"
#include "devices.h"

static const struct device_ops default_ops = {
	.name = "default-ops",
};

dd_list *dev_head;

dd_list *get_device_list_head(void)
{
	return dev_head;
}

void add_device(const struct device_ops *dev)
{
	if (dev->priority == DEVICE_PRIORITY_HIGH)
		DD_LIST_PREPEND(dev_head, dev);
	else
		DD_LIST_APPEND(dev_head, dev);
}

void remove_device(const struct device_ops *dev)
{
	DD_LIST_REMOVE(dev_head, dev);
}

const struct device_ops *find_device(const char *name)
{
	dd_list *elem;
	const struct device_ops *dev;

	DD_LIST_FOREACH(dev_head, elem, dev) {
		if (!strcmp(dev->name, name))
			return dev;
	}

	dev = &default_ops;
	return dev;
}

int check_default(const struct device_ops *dev)
{
	return (dev == &default_ops);
}

GVariant *dbus_device_list(GDBusConnection *conn,
	const gchar *sender, const gchar *path, const gchar *iface, const gchar *name,
	GVariant *param, GDBusMethodInvocation *invocation, gpointer user_data)
{
	dd_list *elem;
	const struct device_ops *dev;

	_I("device list!");
	DD_LIST_FOREACH(dev_head, elem, dev)
		_I("%s", dev->name);

	g_dbus_method_invocation_return_value(invocation, NULL);
	return NULL;
}

static const dbus_method_s dbus_methods[] = {
	{ "DeviceList",          NULL,        NULL,        dbus_device_list },
	/* Add methods here */
};

static const dbus_interface_u dbus_interface = {
	.oh = NULL,
	.name = DEVICED_INTERFACE_CORE,
	.methods = dbus_methods,
	.nr_methods = ARRAY_SIZE(dbus_methods),
};

void devices_init(void *data)
{
	dd_list *elem, *elem_n;
	const struct device_ops *dev;
	int ret;

	DD_LIST_FOREACH_SAFE(dev_head, elem, elem_n, dev) {
		if (dev->probe && dev->probe(data) != 0) {
			_E("[%s] probe fail", dev->name);
			DD_LIST_REMOVE(dev_head, dev);
			continue;
		}

		_D("[%s] initialize", dev->name);
		if (dev->init)
			dev->init(data);
	}

	ret = dbus_handle_add_dbus_object(NULL, DEVICED_PATH_CORE, &dbus_interface);
	if (ret < 0)
		_E("fail to init dbus method(%d)", ret);

	/* register every objects */
	if (dbus_handle_register_dbus_object_all(NULL) < 0)
		_E("Failed to register dbus method! %d", ret);
}

void devices_exit(void *data)
{
	dd_list *elem;
	const struct device_ops *dev;

	DD_LIST_FOREACH(dev_head, elem, dev) {
		_D("[%s] deinitialize", dev->name);
		if (dev->exit)
			dev->exit(data);
	}
}
